@@ -20,12 +20,8 @@
 
 from hy.macros import process as mprocess
 
-import hy.mangle
-
-
 MACROS = [
     "hy.core.bootstrap",
-    "hy.core.mangles",
 ]
 
 
@@ -35,8 +31,6 @@ def process(tree):
     while old != tree:
         old = tree
         tree = mprocess(tree)
-        for m in hy.mangle.MANGLES:
-            m().mangle(tree)
     return tree
 
 
