@@ -33,5 +33,8 @@ class HyList(HyObject, list):
         HyObject.replace(self, other)
         return self
 
+    def __add__(self, other):
+        return self.__class__(super(HyList, self).__add__(other))
+
     def __repr__(self):
         return "[%s]" % (" ".join([str(x) for x in self]))
